/* --COPYRIGHT--,BSD
 * Copyright (c) 2018, Texas Instruments Incorporated
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * *  Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * *  Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * *  Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * --/COPYRIGHT--*/

package com.example.simplelinksdkexplorer;

import android.content.Intent;
import android.graphics.Color;
import android.support.v7.app.AppCompatActivity;
import android.os.Bundle;
import android.support.v7.widget.Toolbar;
import android.view.View;
import android.widget.AdapterView;
import android.widget.ListView;

import com.example.simplelinksdkexplorer.oad.OADActivity;
import com.example.simplelinksdkexplorer.projectzero.ProjectZeroActivity;
import com.example.simplelinksdkexplorer.sensorhub.SensorHubActivity;
import com.example.simplelinksdkexplorer.simpleap.SimpleAPActivity;
import com.example.simplelinksdkexplorer.tempsensor.ThermometerActivity;

public class ProfileExamplesActivity extends AppCompatActivity {

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.activity_profile_examples);

        Toolbar myToolbar = (Toolbar) findViewById(R.id.main_toolbar);
        myToolbar.setBackgroundColor(getResources().getColor(R.color.ti_red));
        myToolbar.setTitleTextColor(Color.WHITE);
        myToolbar.setSubtitleTextColor(Color.WHITE);
        setSupportActionBar(myToolbar);

        // Setup profile examples list
        String[] profile_examples_items = this.getResources().getStringArray(R.array.profile_examples);
        int[] profile_examples_drawableIds = {R.drawable.projectzero, R.drawable.simpleap, R.drawable.tempsens, R.drawable.sensor, R.drawable.oad};
        MenuListAdapter profile_examples_adapter = new MenuListAdapter(this, profile_examples_items, profile_examples_drawableIds);

        ListView profileExamplesListView = (ListView)findViewById(R.id.profile_example_list);
        profileExamplesListView.setAdapter(profile_examples_adapter);

        ListView profileExampleListView = (ListView)findViewById(R.id.profile_example_list);
        profileExampleListView.setOnItemClickListener(new AdapterView.OnItemClickListener() {
            @Override
            public void onItemClick(AdapterView<?> parent, View view, int position, long id) {
                if (position == 0) {
                    Intent intent = new Intent(ProfileExamplesActivity.this, ProjectZeroActivity.class);
                    startActivity(intent);
                }
                else if (position == 1) {
                    Intent intent = new Intent(ProfileExamplesActivity.this, SimpleAPActivity.class);
                    startActivity(intent);
                }
                else if (position == 2) {
                    Intent intent = new Intent(ProfileExamplesActivity.this, ThermometerActivity.class);
                    startActivity(intent);
                }
                else if (position == 3) {
                    Intent intent = new Intent(ProfileExamplesActivity.this, SensorHubActivity.class);
                    startActivity(intent);
                }
                else if (position == 4) {
                    Intent intent = new Intent(ProfileExamplesActivity.this, OADActivity.class);
                    startActivity(intent);
                }
            }
        });
    }
}
