/* --COPYRIGHT--,BSD
 * Copyright (c) 2018, Texas Instruments Incorporated
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * *  Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * *  Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * *  Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * --/COPYRIGHT--*/

package com.example.simplelinksdkexplorer.projectzero;

import android.bluetooth.BluetoothDevice;
import android.bluetooth.BluetoothGattCharacteristic;
import android.media.MediaPlayer;
import android.os.Bundle;
import android.support.v4.app.Fragment;
import android.util.Log;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.ImageView;

import com.example.simplelinksdkexplorer.R;
import com.example.simplelinksdkexplorer.gatt.CharacteristicChangeListener;
import com.example.simplelinksdkexplorer.gatt.GattManager;
import com.example.simplelinksdkexplorer.gatt.GattOperationBundle;
import com.example.simplelinksdkexplorer.gatt.operations.GattSetNotificationOperation;

import java.util.UUID;

public class ProjectZeroButtonFragment extends Fragment {

    private final String PZ_BUTTON_SERVICE_UUID = "f0001120-0451-4000-b000-000000000000";
    private final String PZ_S1_BUTTON_CHAR_UUID = "f0001121-0451-4000-b000-000000000000";
    private final String PZ_S2_BUTTON_CHAR_UUID = "f0001122-0451-4000-b000-000000000000";

    private BluetoothDevice mBtdevice;
    private GattManager mGattManager;

    private ImageView buttonS1;
    private ImageView buttonS2;

    private MediaPlayer mediaPlayer_1khz;
    private MediaPlayer mediaPlayer_400hz;

    public ProjectZeroButtonFragment() {
        // Required empty public constructor
    }

    @Override
    public void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setRetainInstance(true);

        mediaPlayer_1khz = MediaPlayer.create(getContext(), R.raw.tone_1khz);
        mediaPlayer_400hz = MediaPlayer.create(getContext(), R.raw.tone_400hz);

        mediaPlayer_1khz.setLooping(true);
        mediaPlayer_400hz.setLooping(true);
    }

    @Override
    public View onCreateView(LayoutInflater inflater, ViewGroup container,
                             Bundle savedInstanceState) {
        // Inflate the layout for this fragment
        return inflater.inflate(R.layout.fragment_project_zero_button, container, false);
    }

    @Override
    public void onViewCreated(View view, Bundle savedInstanceState) {
        buttonS1 = (ImageView) getActivity().findViewById(R.id.s1_button);
        buttonS2 = (ImageView) getActivity().findViewById(R.id.s2_button);

        if (savedInstanceState!= null) {

        }
        else {
            GattOperationBundle bundle = new GattOperationBundle();

            bundle.addOperation(new GattSetNotificationOperation(
                    mBtdevice, UUID.fromString(PZ_BUTTON_SERVICE_UUID),
                    UUID.fromString(PZ_S2_BUTTON_CHAR_UUID),
                    UUID.fromString("00002902-0000-1000-8000-00805f9b34fb"),
                    true
            ));

            bundle.addOperation(new GattSetNotificationOperation(
                    mBtdevice, UUID.fromString(PZ_BUTTON_SERVICE_UUID),
                    UUID.fromString(PZ_S1_BUTTON_CHAR_UUID),
                    UUID.fromString("00002902-0000-1000-8000-00805f9b34fb"),
                    true
            ));

            mGattManager.queue(bundle);

            mGattManager.addCharacteristicChangeListener(
                    UUID.fromString(PZ_S1_BUTTON_CHAR_UUID),
                    new CharacteristicChangeListener() {
                        @Override
                        public void onCharacteristicChanged(String deviceAddress, BluetoothGattCharacteristic characteristic) {
                            final int lsb = characteristic.getValue()[0] & 0xff;
                            Log.d("onCharacteristicChanged", String.valueOf(lsb));
                            getActivity().runOnUiThread(new Runnable() {
                                @Override
                                public void run() {
                                    if (lsb == 1) {
                                        buttonS1.setImageDrawable(getResources().getDrawable(R.drawable.onbutton_s1));
                                        mediaPlayer_1khz.start();
                                    } else {
                                        buttonS1.setImageDrawable(getResources().getDrawable(R.drawable.offbutton_s1));
                                        mediaPlayer_1khz.pause();
                                        mediaPlayer_1khz.seekTo(0);
                                    }
                                }
                            });
                        }
                    }
            );

            mGattManager.addCharacteristicChangeListener(
                    UUID.fromString(PZ_S2_BUTTON_CHAR_UUID),
                    new CharacteristicChangeListener() {
                        @Override
                        public void onCharacteristicChanged(String deviceAddress, BluetoothGattCharacteristic characteristic) {
                            final int lsb = characteristic.getValue()[0] & 0xff;
                            Log.d("onCharacteristicChanged", String.valueOf(lsb));
                            getActivity().runOnUiThread(new Runnable() {
                                @Override
                                public void run() {
                                    if (lsb == 1) {
                                        buttonS2.setImageDrawable(getResources().getDrawable(R.drawable.onbutton_s2));
                                        mediaPlayer_400hz.start();
                                    } else {
                                        buttonS2.setImageDrawable(getResources().getDrawable(R.drawable.offbutton_s2));
                                        mediaPlayer_400hz.pause();
                                        mediaPlayer_400hz.seekTo(0);
                                    }
                                }
                            });
                        }
                    }
            );
        }
    }

    public void setBluetoothGatt(GattManager manager, BluetoothDevice btdevice) {
        mGattManager = manager;
        mBtdevice = btdevice;
    }

}
