/* --COPYRIGHT--,BSD
 * Copyright (c) 2018, Texas Instruments Incorporated
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * *  Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * *  Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * *  Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * --/COPYRIGHT--*/

package com.example.simplelinksdkexplorer.projectzero;

import android.bluetooth.BluetoothDevice;
import android.bluetooth.BluetoothGattCharacteristic;
import android.os.Bundle;
import android.support.v4.app.Fragment;
import android.text.method.ScrollingMovementMethod;
import android.util.Log;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.AdapterView;
import android.widget.ArrayAdapter;
import android.widget.Button;
import android.widget.EditText;
import android.widget.Spinner;
import android.widget.TextView;

import com.example.simplelinksdkexplorer.R;
import com.example.simplelinksdkexplorer.gatt.CharacteristicChangeListener;
import com.example.simplelinksdkexplorer.gatt.GattCharacteristicReadCallback;
import com.example.simplelinksdkexplorer.gatt.GattManager;
import com.example.simplelinksdkexplorer.gatt.operations.GattCharacteristicReadOperation;
import com.example.simplelinksdkexplorer.gatt.operations.GattCharacteristicWriteOperation;
import com.example.simplelinksdkexplorer.gatt.operations.GattSetNotificationOperation;

import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.UUID;

public class ProjectZeroDataFragment extends Fragment {

    private final String PZ_DATA_SERVICE_UUID = "f0001130-0451-4000-b000-000000000000";
    private final String PZ_DATA_CHAR_0_UUID = "f0001131-0451-4000-b000-000000000000";
    private final String PZ_DATA_CHAR_1_UUID = "f0001132-0451-4000-b000-000000000000";

    private BluetoothDevice mBtdevice;
    private GattManager mGattManager;

    private TextView log_textview;
    private Spinner char_spinner;
    private EditText write_edittext;
    private Button write_button;

    public ProjectZeroDataFragment() {
        // Required empty public constructor
    }

    @Override
    public void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setRetainInstance(true);
    }

    @Override
    public View onCreateView(LayoutInflater inflater, ViewGroup container,
                             Bundle savedInstanceState) {
        // Inflate the layout for this fragment
        return inflater.inflate(R.layout.fragment_project_zero_data, container, false);
    }

    @Override
    public void onViewCreated(View view, Bundle savedInstanceState) {

        log_textview = (TextView) getActivity().findViewById(R.id.log_textview);
        log_textview.setMovementMethod(new ScrollingMovementMethod());
        write_edittext = (EditText) getActivity().findViewById(R.id.write_edittext);
        write_button = (Button) getActivity().findViewById(R.id.write_button);
        char_spinner = (Spinner) getActivity().findViewById(R.id.char_spinner);

        ArrayAdapter<CharSequence> adapter = ArrayAdapter.createFromResource(getContext(),
                R.array.char_spinner_array, android.R.layout.simple_spinner_item);
        // Specify the layout to use when the list of choices appears
        adapter.setDropDownViewResource(android.R.layout.simple_spinner_dropdown_item);
        // Apply the adapter to the spinner
        char_spinner.setAdapter(adapter);
        char_spinner.setOnItemSelectedListener(new AdapterView.OnItemSelectedListener() {
            @Override
            public void onItemSelected(AdapterView<?> parent, View view, int position, long id) {
                switch (position) {
                    case 0:
                        break;
                    case 1:
                        break;
                }
            }

            @Override
            public void onNothingSelected(AdapterView<?> parent) {
            }
        });

        final SimpleDateFormat dateFormat = new SimpleDateFormat("HH:mm:ss");

        if (savedInstanceState!= null) {

        }
        else {
            mGattManager.queue(new GattSetNotificationOperation(
                    mBtdevice, UUID.fromString(PZ_DATA_SERVICE_UUID),
                    UUID.fromString(PZ_DATA_CHAR_1_UUID),
                    UUID.fromString("00002902-0000-1000-8000-00805f9b34fb"),
                    true
            ));

            mGattManager.addCharacteristicChangeListener(
                    UUID.fromString(PZ_DATA_CHAR_1_UUID),
                    new CharacteristicChangeListener() {
                        @Override
                        public void onCharacteristicChanged(String deviceAddress, final BluetoothGattCharacteristic characteristic) {
                            Log.d("onCharacteristicChanged", characteristic.getStringValue(0));
                            getActivity().runOnUiThread(new Runnable() {
                                @Override
                                public void run() {
                                    addMessage(dateFormat.format(new Date()) + ": Value of " + characteristic.getStringValue(0) + " NOTIFIED\n");
                                }
                            });
                        }
                    }
            );
        }

        write_button.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View view) {
                Log.d("PZ","Write Button Clicked");
                String write_string = write_edittext.getText().toString();

                if (char_spinner.getSelectedItem().toString().equals("Write String")) {
                    mGattManager.queue(new GattCharacteristicWriteOperation(mBtdevice, UUID.fromString(PZ_DATA_SERVICE_UUID), UUID.fromString(PZ_DATA_CHAR_0_UUID), write_string.getBytes()));
                    addMessage(dateFormat.format(new Date()) + ": Value of " + write_string + " written to WRITE characteristic\n");

                    mGattManager.queue(new GattCharacteristicReadOperation(mBtdevice, UUID.fromString(PZ_DATA_SERVICE_UUID), UUID.fromString(PZ_DATA_CHAR_0_UUID), new GattCharacteristicReadCallback() {
                        @Override
                        public void call(final byte[] characteristic) {
                            getActivity().runOnUiThread(new Runnable() {
                                @Override
                                public void run() {
                                    addMessage(dateFormat.format(new Date()) + ": Value of " + new String(characteristic) + " READ\n");
                                }
                            });
                        }
                    }));
                }
                else {
                    mGattManager.queue(new GattCharacteristicWriteOperation(mBtdevice, UUID.fromString(PZ_DATA_SERVICE_UUID), UUID.fromString(PZ_DATA_CHAR_1_UUID), write_string.getBytes()));
                    addMessage(dateFormat.format(new Date()) + ": Value of " + write_string + " written to WRITE_NO_RESP characteristic\n");
                }
            }
        });
    }

    public void setBluetoothGatt(GattManager manager, BluetoothDevice btdevice) {
        mGattManager = manager;
        mBtdevice = btdevice;
    }

    private void addMessage(String msg) {
        // append the new string
        log_textview.append(msg);

        // Scroll to the bottom of textview
        final int scrollAmount = log_textview.getLayout().getLineTop(log_textview.getLineCount()) - log_textview.getHeight();
        if (scrollAmount > 0)
            log_textview.scrollTo(0, scrollAmount);
        else
            log_textview.scrollTo(0, 0);
    }

}
