/* --COPYRIGHT--,BSD
 * Copyright (c) 2018, Texas Instruments Incorporated
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * *  Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * *  Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * *  Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * --/COPYRIGHT--*/

package com.example.simplelinksdkexplorer.projectzero;

import android.bluetooth.BluetoothDevice;
import android.content.Context;
import android.os.Bundle;
import android.support.design.widget.TabLayout;
import android.support.v4.app.Fragment;
import android.support.v4.app.FragmentManager;
import android.support.v4.app.FragmentPagerAdapter;
import android.support.v4.app.FragmentStatePagerAdapter;
import android.support.v7.app.AppCompatActivity;
import android.util.Log;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;

import com.example.simplelinksdkexplorer.CustomViewPager;
import com.example.simplelinksdkexplorer.R;
import com.example.simplelinksdkexplorer.gatt.GattManager;

import java.util.ArrayList;
import java.util.List;

public class ProjectZeroFragment extends Fragment {

    private final String PZ_LED_COLOR_SERVICE_UUID = "f0001110-0451-4000-b000-000000000000";

    private static final String BT_DEVICE = "bt_device";

    private BluetoothDevice mBtdevice;
    private GattManager mGattManager;

    private ProjectZeroFragment.SectionsPagerAdapter mSectionsPagerAdapter;
    private CustomViewPager mViewPager;

    private ProjectZeroLedFragment projectZeroLedFragment;
    private ProjectZeroButtonFragment projectZeroButtonFragment;
    private ProjectZeroDataFragment projectZeroDataFragment;

    public ProjectZeroFragment() {
        // Required empty public constructor
    }

    public static ProjectZeroFragment newInstance(BluetoothDevice btdevice) {
        ProjectZeroFragment fragment = new ProjectZeroFragment();
        Bundle args = new Bundle();
        args.putParcelable(BT_DEVICE, btdevice);
        fragment.setArguments(args);
        return fragment;
    }

    @Override
    public void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setRetainInstance(true);

        if (savedInstanceState!= null) {

        }
        else {
            Log.d("ProjectZeroFragment", "Recreating Everything");
            if (getArguments() != null) {
                mBtdevice = getArguments().getParcelable(BT_DEVICE);
            }

            mGattManager = new GattManager(getActivity(), PZ_LED_COLOR_SERVICE_UUID);
            mGattManager.connectToDevice(mBtdevice, false);

            // Create Tabs for Project Zero demo
            projectZeroLedFragment = new ProjectZeroLedFragment();
            projectZeroLedFragment.setBluetoothGatt(mGattManager, mBtdevice);
            projectZeroButtonFragment = new ProjectZeroButtonFragment();
            projectZeroButtonFragment.setBluetoothGatt(mGattManager, mBtdevice);
            projectZeroDataFragment = new ProjectZeroDataFragment();
            projectZeroDataFragment.setBluetoothGatt(mGattManager, mBtdevice);

            Log.d("ProjectZeroFragment", "onCreate");
        }
    }

    @Override
    public View onCreateView(LayoutInflater inflater, ViewGroup container,
                             Bundle savedInstanceState) {
        // Inflate the layout for this fragment
        return inflater.inflate(R.layout.fragment_project_zero, container, false);
    }

    @Override
    public void onViewCreated(View view, Bundle savedInstanceState) {
        // Create the adapter that will return a fragment for each of the three
        // primary sections of the activity.
        List<Fragment> fragmentList = new ArrayList<>();
        fragmentList.add(projectZeroLedFragment);
        fragmentList.add(projectZeroButtonFragment);
        fragmentList.add(projectZeroDataFragment);

        mSectionsPagerAdapter = new ProjectZeroFragment.SectionsPagerAdapter(getChildFragmentManager(), fragmentList);

        // Set up the ViewPager with the sections adapter.
        mViewPager = (CustomViewPager) getActivity().findViewById(R.id.project_zero_viewpager);
        mViewPager.setAdapter(mSectionsPagerAdapter);

        final TabLayout tablayout = (TabLayout) getActivity().findViewById(R.id.project_zero_tablayout);
        tablayout.setupWithViewPager(mViewPager);
    }

    @Override
    public void onAttach(Context context) {
        super.onAttach(context);
    }

    @Override
    public void onDestroy() {
        Log.d("ProjectZeroFragment", "onDestroy()");
        super.onDestroy();
        if (mGattManager!= null) {
            mGattManager.close(mBtdevice);
            mGattManager = null;
        }
    }

    @Override
    public void onDetach() {
        super.onDetach();
    }

    @Override
    public void onResume() {
        super.onResume();
        ((AppCompatActivity)getActivity()).getSupportActionBar().setTitle("Project Zero");
    }

    @Override
    public void onPause() {
        super.onPause();
    }

    /**
     * A {@link FragmentPagerAdapter} that returns a fragment corresponding to
     * one of the sections/tabs/pages.
     */
    public class SectionsPagerAdapter extends FragmentStatePagerAdapter {

        private List<Fragment> fragmentList;

        public SectionsPagerAdapter(FragmentManager fm, List<Fragment> fl) {
            super(fm);
            fragmentList = fl;
        }

        @Override
        public Fragment getItem(int position) {
            // getItem is called to instantiate the fragment for the given page.
            return fragmentList.get(position);
        }

        @Override
        public int getCount() {
            return fragmentList.size();
        }

        @Override
        public CharSequence getPageTitle(int position) {
            switch (position) {
                case 0:
                    return "LED";
                case 1:
                    return "Button";
                case 2:
                    return "Data";
                default:
                    return "";
            }
        }
    }
}
