/* --COPYRIGHT--,BSD
 * Copyright (c) 2018, Texas Instruments Incorporated
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * *  Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * *  Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * *  Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * --/COPYRIGHT--*/

package com.example.simplelinksdkexplorer.projectzero;

import android.bluetooth.BluetoothDevice;
import android.os.Bundle;
import android.support.v4.app.Fragment;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.FrameLayout;
import android.widget.LinearLayout;

import com.example.simplelinksdkexplorer.CustomViewPager;
import com.example.simplelinksdkexplorer.R;
import com.example.simplelinksdkexplorer.gatt.GattManager;
import com.example.simplelinksdkexplorer.gatt.operations.GattCharacteristicWriteOperation;

import java.util.UUID;

public class ProjectZeroLedFragment extends Fragment {

    private final String PZ_LED_COLOR_SERVICE_UUID = "f0001110-0451-4000-b000-000000000000";
    private final String PZ_LED_COLOR_CHAR_UUID = "f0001111-0451-4000-b000-000000000000";

    private BluetoothDevice mBtdevice;
    private GattManager mGattManager;

    private HSVColorWheel colorWheel;
    private HSVValueSlider valueSlider;
    private View selectedColorView;
    private int selectedColor;

    private boolean tx_ready_flag;

    public ProjectZeroLedFragment() {
        // Required empty public constructor
    }

    @Override
    public void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setRetainInstance(true);

        if (savedInstanceState != null) {

        } else {
            selectedColor = 0xFFCCCCCC;
            tx_ready_flag = true;

            colorWheel = new HSVColorWheel(getActivity());
            colorWheel.setListener(new HSVColorWheel.OnColorSelectedListener() {
                @Override
                public void wheelColorSelected(Integer color) {
                    valueSlider.setColor(color, true);
                }
                @Override
                public void setSwipeable(boolean swipeable) {
                    ((CustomViewPager)getActivity().findViewById(R.id.project_zero_viewpager)).setSwipeable(swipeable);
                }
            });

            valueSlider = new HSVValueSlider(getActivity());
            valueSlider.setColor(0xFFCCCCCC, false);

            selectedColorView = new View(getContext());
            selectedColorView.setBackgroundColor(selectedColor);
        }
    }

    @Override
    public View onCreateView(LayoutInflater inflater, ViewGroup container,
                             Bundle savedInstanceState) {
        // Inflate the layout for this fragment
        return inflater.inflate(R.layout.fragment_project_zero_led, container, false);
    }

    @Override
    public void onViewCreated(View view, Bundle savedInstanceState) {
        // Setup color wheel
        if (colorWheel.getParent() != null)
            ((ViewGroup)colorWheel.getParent()).removeView(colorWheel);
        LinearLayout color_wheel_layout = (LinearLayout) getActivity().findViewById(R.id.color_wheel_container);
        color_wheel_layout.setPaddingRelative(150, 20, 150, 20);
        LinearLayout.LayoutParams lp = new LinearLayout.LayoutParams( LinearLayout.LayoutParams.WRAP_CONTENT, LinearLayout.LayoutParams.WRAP_CONTENT );
        color_wheel_layout.addView(colorWheel, lp);

        // Setup color slider
        if (valueSlider.getParent() != null)
            ((ViewGroup)valueSlider.getParent()).removeView(valueSlider);
        LinearLayout value_slider_layout = (LinearLayout) getActivity().findViewById(R.id.value_slider_container);
        value_slider_layout.setPaddingRelative(150, 5, 150, 5);
        int selectedColorHeight = (int) (getActivity().getResources().getDisplayMetrics().density * 40);
        lp = new LinearLayout.LayoutParams( FrameLayout.LayoutParams.WRAP_CONTENT, selectedColorHeight + 2 * 1 );
        value_slider_layout.addView( valueSlider, lp );

        // Setup selected color box
        if (selectedColorView.getParent() != null)
            ((ViewGroup)selectedColorView.getParent()).removeView(selectedColorView);
        LinearLayout selectedColorViewLayout = (LinearLayout) getActivity().findViewById(R.id.selected_color_container);
        selectedColorViewLayout.setPaddingRelative(150, 5, 150, 5);
        lp = new LinearLayout.LayoutParams(LinearLayout.LayoutParams.WRAP_CONTENT, selectedColorHeight + 2);
        selectedColorViewLayout.addView(selectedColorView, lp);

        valueSlider.setListener(new HSVValueSlider.OnColorSelectedListener() {
            @Override
            public void sliderColorSelected(Integer color) {
                selectedColorView.setBackgroundColor(color);
                String color_string = Integer.toHexString(color);
                byte[] value = hexStringToByteArray(color_string.substring(2, color_string.length()));
                if (tx_ready_flag) {
                    mGattManager.queue(new GattCharacteristicWriteOperation(mBtdevice, UUID.fromString(PZ_LED_COLOR_SERVICE_UUID), UUID.fromString(PZ_LED_COLOR_CHAR_UUID), value));
                    tx_ready_flag = false;
                    // Throttle LED color write frequency to 50Hz to prevent embedded device code from locking up
                    new android.os.Handler().postDelayed(
                            new Runnable() {
                                public void run() {
                                    tx_ready_flag = true;
                                }
                            },
                            20
                    );
                }
            }
            @Override
            public void setSwipeable(boolean swipeable) {
                ((CustomViewPager)getActivity().findViewById(R.id.project_zero_viewpager)).setSwipeable(swipeable);
            }
        });
    }

    @Override
    public void onResume() {
        super.onResume();
    }

    public void setBluetoothGatt(GattManager manager, BluetoothDevice btdevice) {
        mGattManager = manager;
        mBtdevice = btdevice;
    }

    public static byte[] hexStringToByteArray(String s) {
        int len = s.length();
        byte[] data = new byte[len/2];
        for (int i = 0; i < len; i += 2) {
            data[i/2] = (byte) ((Character.digit(s.charAt(i), 16) << 4)
                    + Character.digit(s.charAt(i+1), 16));
        }
        return data;
    }
}
