/* --COPYRIGHT--,BSD
 * Copyright (c) 2018, Texas Instruments Incorporated
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * *  Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * *  Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * *  Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * --/COPYRIGHT--*/

package com.example.simplelinksdkexplorer.sensorhub;

import android.bluetooth.BluetoothDevice;
import android.graphics.Color;
import android.os.Bundle;
import android.support.v4.app.Fragment;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.CompoundButton;
import android.widget.SeekBar;
import android.widget.Switch;
import android.widget.TextView;

import com.github.mikephil.charting.charts.LineChart;
import com.github.mikephil.charting.components.XAxis;
import com.github.mikephil.charting.data.Entry;
import com.github.mikephil.charting.data.LineData;
import com.github.mikephil.charting.data.LineDataSet;
import com.github.mikephil.charting.interfaces.datasets.ILineDataSet;
import com.example.simplelinksdkexplorer.R;
import com.example.simplelinksdkexplorer.gatt.GattManager;
import com.example.simplelinksdkexplorer.gatt.GattOperationBundle;
import com.example.simplelinksdkexplorer.gatt.operations.GattCharacteristicWriteOperation;

import java.nio.ByteBuffer;
import java.nio.ByteOrder;
import java.util.ArrayList;
import java.util.List;
import java.util.UUID;

public class SensorHubAccelerometerFragment extends Fragment {

    private final String SENSHUB_MOVE_SERVICE_UUID = "f000aa80-0451-4000-b000-000000000000";
    private final String SENSHUB_MOVE_CONF_CHAR_UUID = "f000aa82-0451-4000-b000-000000000000";
    private final String SENSHUB_MOVE_PERI_CHAR_UUID = "f000aa83-0451-4000-b000-000000000000";

    private BluetoothDevice mBtdevice;
    private GattManager mGattManager;

    private int curIndex;
    private final int maxNumOfReadings = 20;

    private LineChart chart;
    private List<Entry> accel_x_entries;
    private List<Entry> accel_y_entries;
    private List<Entry> accel_z_entries;
    private LineDataSet accel_x_dataSet;
    private LineDataSet accel_y_dataSet;
    private LineDataSet accel_z_dataSet;
    private LineData lineData;

    private TextView accel_x_textview;
    private TextView accel_y_textview;
    private TextView accel_z_textview;

    private Switch notification_switch;
    private SeekBar period_seekbar;
    private TextView period_textview;

    public SensorHubAccelerometerFragment() {
        // Required empty public constructor
    }

    @Override
    public void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setRetainInstance(true);

        if (savedInstanceState != null) {

        } else {
            initializeData();
        }
    }

    @Override
    public View onCreateView(LayoutInflater inflater, ViewGroup container,
                             Bundle savedInstanceState) {
        // Inflate the layout for this fragment
        return inflater.inflate(R.layout.fragment_sensor_hub_accelerometer, container, false);
    }

    @Override
    public void onViewCreated(View view, Bundle savedInstanceState) {
        initializeChart();

        notification_switch = (Switch)getActivity().findViewById(R.id.accel_notification_switch);
        notification_switch.setChecked(true);
        notification_switch.setOnCheckedChangeListener(new CompoundButton.OnCheckedChangeListener() {
            @Override
            public void onCheckedChanged(CompoundButton buttonView, boolean isChecked) {
                GattOperationBundle bundle = new GattOperationBundle();
                if (isChecked) {
                    initializeData();
                    initializeChart();
                    mGattManager.queue(new GattCharacteristicWriteOperation(
                            mBtdevice, UUID.fromString(SENSHUB_MOVE_SERVICE_UUID),
                            UUID.fromString(SENSHUB_MOVE_CONF_CHAR_UUID),
                            new byte[]{0x01}
                    ));
                }
                else {
                    mGattManager.queue(new GattCharacteristicWriteOperation(
                            mBtdevice, UUID.fromString(SENSHUB_MOVE_SERVICE_UUID),
                            UUID.fromString(SENSHUB_MOVE_CONF_CHAR_UUID),
                            new byte[]{0x00}
                    ));
                }
            }
        });

        period_seekbar = (SeekBar) getActivity().findViewById(R.id.accel_period_seekbar);
        period_textview = (TextView) getActivity().findViewById(R.id.accel_period_textview);

        period_seekbar.setOnSeekBarChangeListener(new SeekBar.OnSeekBarChangeListener() {
            @Override
            public void onProgressChanged(SeekBar seekBar, int progress, boolean fromUser) {
                mGattManager.queue(new GattCharacteristicWriteOperation(
                        mBtdevice,
                        UUID.fromString(SENSHUB_MOVE_SERVICE_UUID),
                        UUID.fromString(SENSHUB_MOVE_PERI_CHAR_UUID),
                        ByteBuffer.allocate(2).order(ByteOrder.LITTLE_ENDIAN).putShort((short)((progress+1)*100)).array()
                ));
                period_textview.setText(String.format("%.1f", (progress+1)/10.0f) + "s");
            }

            @Override
            public void onStartTrackingTouch(SeekBar seekBar) {

            }

            @Override
            public void onStopTrackingTouch(SeekBar seekBar) {

            }
        });

        accel_x_textview = (TextView) getActivity().findViewById(R.id.accel_x_textview);
        accel_y_textview = (TextView) getActivity().findViewById(R.id.accel_y_textview);
        accel_z_textview = (TextView) getActivity().findViewById(R.id.accel_z_textview);
    }


    public void setBluetoothGatt(GattManager manager, BluetoothDevice btdevice) {
        mGattManager = manager;
        mBtdevice = btdevice;
    }

    private void initializeChart() {
        chart = (LineChart) getActivity().findViewById(R.id.accel_chart);
        chart.getXAxis().setDrawGridLines(false);
        chart.getXAxis().setDrawLabels(false);
        chart.getXAxis().setPosition(XAxis.XAxisPosition.BOTH_SIDED);

        chart.getAxisLeft().setAxisMaximum(32768);
        chart.getAxisLeft().setAxisMinimum(-32768);

        chart.getAxisRight().setDrawGridLines(false);
        chart.getAxisRight().setDrawLabels(false);

        chart.getLegend().setEnabled(true);
        chart.getDescription().setText("Accelerometer Data from Sensors BoosterPack");
        chart.setTouchEnabled(false);
        chart.setData(lineData);
    }

    private void initializeData() {
        curIndex = 0;
        accel_x_entries = new ArrayList<Entry>();
        accel_y_entries = new ArrayList<Entry>();
        accel_z_entries = new ArrayList<Entry>();
        accel_x_dataSet = new LineDataSet(accel_x_entries, "Accel_X");
        accel_y_dataSet = new LineDataSet(accel_y_entries, "Accel_Y");
        accel_z_dataSet = new LineDataSet(accel_z_entries, "Accel_Z");
        List<ILineDataSet> dataSets = new ArrayList<ILineDataSet>();
        dataSets.add(accel_x_dataSet);
        dataSets.add(accel_y_dataSet);
        dataSets.add(accel_z_dataSet);
        lineData = new LineData(dataSets);

        accel_x_dataSet.setDrawValues(false);
        accel_x_dataSet.setColor(Color.parseColor("Red"));
        accel_x_dataSet.setCircleColor(Color.parseColor("Red"));
        accel_x_dataSet.setDrawCircleHole(false);

        accel_y_dataSet.setDrawValues(false);
        accel_y_dataSet.setColor(Color.parseColor("Green"));
        accel_y_dataSet.setCircleColor(Color.parseColor("Green"));
        accel_y_dataSet.setDrawCircleHole(false);

        accel_z_dataSet.setDrawValues(false);
        accel_z_dataSet.setColor(Color.parseColor("Blue"));
        accel_z_dataSet.setCircleColor(Color.parseColor("Blue"));
        accel_z_dataSet.setDrawCircleHole(false);
    }

    public void appendDataToChart(short[] movement) {

        short accel_x = movement[3];
        short accel_y = movement[4];
        short accel_z = movement[5];

        if (curIndex < maxNumOfReadings)
            curIndex++;
        else {
            for (Entry entry : accel_x_dataSet.getValues())
                entry.setX(entry.getX()-1);
            accel_x_dataSet.removeFirst();
            for (Entry entry : accel_y_dataSet.getValues())
                entry.setX(entry.getX()-1);
            accel_y_dataSet.removeFirst();
            for (Entry entry : accel_z_dataSet.getValues())
                entry.setX(entry.getX()-1);
            accel_z_dataSet.removeFirst();
        }
        accel_x_dataSet.addEntry(new Entry(curIndex-1, accel_x));
        accel_y_dataSet.addEntry(new Entry(curIndex-1, accel_y));
        accel_z_dataSet.addEntry(new Entry(curIndex-1, accel_z));

        lineData.notifyDataChanged();
        accel_x_dataSet.setDrawValues(true);
        accel_y_dataSet.setDrawValues(true);
        accel_z_dataSet.setDrawValues(true);
        chart.notifyDataSetChanged();
        chart.invalidate();

        accel_x_textview.setText(String.format("%d",accel_x));
        accel_y_textview.setText(String.format("%d",accel_y));
        accel_z_textview.setText(String.format("%d",accel_z));
    }

}
