/* --COPYRIGHT--,BSD
 * Copyright (c) 2018, Texas Instruments Incorporated
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * *  Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * *  Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * *  Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * --/COPYRIGHT--*/

package com.example.simplelinksdkexplorer.sensorhub;

import android.bluetooth.BluetoothDevice;
import android.bluetooth.BluetoothGattCharacteristic;
import android.content.Context;
import android.os.Bundle;
import android.support.design.widget.TabLayout;
import android.support.v4.app.Fragment;
import android.support.v4.app.FragmentManager;
import android.support.v4.app.FragmentPagerAdapter;
import android.support.v4.app.FragmentStatePagerAdapter;
import android.support.v7.app.AppCompatActivity;
import android.util.Log;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;

import com.example.simplelinksdkexplorer.CustomViewPager;
import com.example.simplelinksdkexplorer.R;
import com.example.simplelinksdkexplorer.gatt.CharacteristicChangeListener;
import com.example.simplelinksdkexplorer.gatt.GattManager;
import com.example.simplelinksdkexplorer.gatt.operations.GattCharacteristicWriteOperation;
import com.example.simplelinksdkexplorer.gatt.operations.GattSetNotificationOperation;

import java.nio.ByteBuffer;
import java.nio.ByteOrder;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;
import java.util.UUID;

public class SensorHubFragment extends Fragment {

    private final String SENSHUB_IRTEMP_SERVICE_UUID = "f000aa00-0451-4000-b000-000000000000";
    private final String SENSHUB_IRTEMP_DATA_CHAR_UUID = "f000aa01-0451-4000-b000-000000000000";
    private final String SENSHUB_IRTEMP_CONF_CHAR_UUID = "f000aa02-0451-4000-b000-000000000000";
    private final String SENSHUB_IRTEMP_PERI_CHAR_UUID = "f000aa03-0451-4000-b000-000000000000";
    private final String SENSHUB_HUMIDITY_SERVICE_UUID = "f000aa20-0451-4000-b000-000000000000";
    private final String SENSHUB_HUMIDITY_DATA_CHAR_UUID = "f000aa21-0451-4000-b000-000000000000";
    private final String SENSHUB_HUMIDITY_CONF_CHAR_UUID = "f000aa22-0451-4000-b000-000000000000";
    private final String SENSHUB_HUMIDITY_PERI_CHAR_UUID = "f000aa23-0451-4000-b000-000000000000";
    private final String SENSHUB_BAROMETER_SERVICE_UUID = "f000aa40-0451-4000-b000-000000000000";
    private final String SENSHUB_BAROMETER_DATA_CHAR_UUID = "f000aa41-0451-4000-b000-000000000000";
    private final String SENSHUB_BAROMETER_CONF_CHAR_UUID = "f000aa42-0451-4000-b000-000000000000";
    private final String SENSHUB_BAROMETER_PERI_CHAR_UUID = "f000aa44-0451-4000-b000-000000000000";
    private final String SENSHUB_OPTIC_SERVICE_UUID = "f000aa70-0451-4000-b000-000000000000";
    private final String SENSHUB_OPTIC_DATA_CHAR_UUID = "f000aa71-0451-4000-b000-000000000000";
    private final String SENSHUB_OPTIC_CONF_CHAR_UUID = "f000aa72-0451-4000-b000-000000000000";
    private final String SENSHUB_OPTIC_PERI_CHAR_UUID = "f000aa73-0451-4000-b000-000000000000";
    private final String SENSHUB_MOVE_SERVICE_UUID = "f000aa80-0451-4000-b000-000000000000";
    private final String SENSHUB_MOVE_DATA_CHAR_UUID = "f000aa81-0451-4000-b000-000000000000";
    private final String SENSHUB_MOVE_CONF_CHAR_UUID = "f000aa82-0451-4000-b000-000000000000";
    private final String SENSHUB_MOVE_PERI_CHAR_UUID = "f000aa83-0451-4000-b000-000000000000";

    private static final String BT_DEVICE = "bt_device";

    private BluetoothDevice mBtdevice;
    private GattManager mGattManager;

    private SectionsPagerAdapter mSectionsPagerAdapter;
    private CustomViewPager mViewPager;

    private SensorHubOverviewFragment sensorHubOverviewFragment;
    private SensorHubThermometerFragment sensorHubThermometerFragment;
    private SensorHubHumidityFragment sensorHubHumidityFragment;
    private SensorHubOpticalFragment sensorHubOpticalFragment;
    private SensorHubPressureFragment sensorHubPressureFragment;
    private SensorHubAccelerometerFragment sensorHubAccelerometerFragment;
    private SensorHubGyroFragment sensorHubGyroFragment;
    private SensorHubMagnetometerFragment sensorHubMagnetometerFragment;

    public SensorHubFragment() {
        // Required empty public constructor
    }

    public static SensorHubFragment newInstance(BluetoothDevice btdevice) {
        SensorHubFragment fragment = new SensorHubFragment();
        Bundle args = new Bundle();
        args.putParcelable(BT_DEVICE, btdevice);
        fragment.setArguments(args);
        return fragment;
    }

    @Override
    public void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setRetainInstance(true);

        if (savedInstanceState!= null) {

        }
        else {
            if (getArguments() != null) {
                mBtdevice = getArguments().getParcelable(BT_DEVICE);
            }

            mGattManager = new GattManager(getActivity(), SENSHUB_IRTEMP_SERVICE_UUID);
            mGattManager.connectToDevice(mBtdevice, false);

            sensorHubOverviewFragment = new SensorHubOverviewFragment();
            sensorHubThermometerFragment = new SensorHubThermometerFragment();
            sensorHubThermometerFragment.setBluetoothGatt(mGattManager, mBtdevice);
            sensorHubHumidityFragment = new SensorHubHumidityFragment();
            sensorHubHumidityFragment.setBluetoothGatt(mGattManager, mBtdevice);
            sensorHubOpticalFragment = new SensorHubOpticalFragment();
            sensorHubOpticalFragment.setBluetoothGatt(mGattManager, mBtdevice);
            sensorHubPressureFragment = new SensorHubPressureFragment();
            sensorHubPressureFragment.setBluetoothGatt(mGattManager, mBtdevice);
            sensorHubAccelerometerFragment = new SensorHubAccelerometerFragment();
            sensorHubAccelerometerFragment.setBluetoothGatt(mGattManager, mBtdevice);
            sensorHubGyroFragment = new SensorHubGyroFragment();
            sensorHubGyroFragment.setBluetoothGatt(mGattManager, mBtdevice);
            sensorHubMagnetometerFragment = new SensorHubMagnetometerFragment();
            sensorHubMagnetometerFragment.setBluetoothGatt(mGattManager, mBtdevice);
        }
    }

    @Override
    public View onCreateView(LayoutInflater inflater, ViewGroup container,
                             Bundle savedInstanceState) {
        // Inflate the layout for this fragment
        return inflater.inflate(R.layout.fragment_sensor_hub, container, false);
    }

    @Override
    public void onViewCreated(View view, Bundle savedInstanceState) {
        List<Fragment> fragmentList = new ArrayList<>();

        fragmentList.add(sensorHubOverviewFragment);
        fragmentList.add(sensorHubThermometerFragment);
        fragmentList.add(sensorHubHumidityFragment);
        fragmentList.add(sensorHubOpticalFragment);
        fragmentList.add(sensorHubPressureFragment);
        fragmentList.add(sensorHubAccelerometerFragment);
        fragmentList.add(sensorHubGyroFragment);
        fragmentList.add(sensorHubMagnetometerFragment);

        // Create the adapter that will return a fragment each tab.
        mSectionsPagerAdapter = new SensorHubFragment.SectionsPagerAdapter(getChildFragmentManager(), fragmentList);

        // Set up the ViewPager with the sections adapter.
        mViewPager = (CustomViewPager) getActivity().findViewById(R.id.sensor_hub_viewpager);
        mViewPager.setAdapter(mSectionsPagerAdapter);
        mViewPager.setOffscreenPageLimit(7);

        final TabLayout tablayout = (TabLayout) getActivity().findViewById(R.id.sensor_hub_tablayout);
        tablayout.setupWithViewPager(mViewPager);

        if (savedInstanceState!= null) {

        }
        else {
            // Setup IR Temperature
            mGattManager.queue(new GattCharacteristicWriteOperation(
                    mBtdevice,
                    UUID.fromString(SENSHUB_IRTEMP_SERVICE_UUID),
                    UUID.fromString(SENSHUB_IRTEMP_PERI_CHAR_UUID),
                    new byte[]{0x64}
            ));
            mGattManager.queue(new GattCharacteristicWriteOperation(
                    mBtdevice, UUID.fromString(SENSHUB_IRTEMP_SERVICE_UUID),
                    UUID.fromString(SENSHUB_IRTEMP_CONF_CHAR_UUID),
                    new byte[]{0x01}
            ));
            mGattManager.queue(new GattSetNotificationOperation(
                    mBtdevice,
                    UUID.fromString(SENSHUB_IRTEMP_SERVICE_UUID),
                    UUID.fromString(SENSHUB_IRTEMP_DATA_CHAR_UUID),
                    UUID.fromString("00002902-0000-1000-8000-00805f9b34fb"),
                    true
            ));

            mGattManager.addCharacteristicChangeListener(
                    UUID.fromString(SENSHUB_IRTEMP_DATA_CHAR_UUID),
                    new CharacteristicChangeListener() {
                        @Override
                        public void onCharacteristicChanged(String deviceAddress, BluetoothGattCharacteristic characteristic) {
                            final float temp = ByteBuffer.wrap(characteristic.getValue()).order(ByteOrder.LITTLE_ENDIAN).getFloat();
                            getActivity().runOnUiThread(new Runnable() {
                                @Override
                                public void run() {
                                    sensorHubOverviewFragment.updateIRTemp(temp);
                                    if (sensorHubThermometerFragment.getView() != null)
                                        sensorHubThermometerFragment.appendDataToChart(temp);
                                }
                            });
                        }
                    }
            );

            // Setup Humidity
            mGattManager.queue(new GattCharacteristicWriteOperation(
                    mBtdevice,
                    UUID.fromString(SENSHUB_HUMIDITY_SERVICE_UUID),
                    UUID.fromString(SENSHUB_HUMIDITY_PERI_CHAR_UUID),
                    new byte[]{0x64}
            ));
            mGattManager.queue(new GattCharacteristicWriteOperation(
                    mBtdevice,
                    UUID.fromString(SENSHUB_HUMIDITY_SERVICE_UUID),
                    UUID.fromString(SENSHUB_HUMIDITY_CONF_CHAR_UUID),
                    new byte[]{0x01}
            ));
            mGattManager.queue(new GattSetNotificationOperation(
                    mBtdevice,
                    UUID.fromString(SENSHUB_HUMIDITY_SERVICE_UUID),
                    UUID.fromString(SENSHUB_HUMIDITY_DATA_CHAR_UUID),
                    UUID.fromString("00002902-0000-1000-8000-00805f9b34fb"),
                    true
            ));

            mGattManager.addCharacteristicChangeListener(
                    UUID.fromString(SENSHUB_HUMIDITY_DATA_CHAR_UUID),
                    new CharacteristicChangeListener() {
                        @Override
                        public void onCharacteristicChanged(String deviceAddress, BluetoothGattCharacteristic characteristic) {
                            final float humidity = ByteBuffer.wrap(characteristic.getValue()).order(ByteOrder.LITTLE_ENDIAN).getFloat();
                            getActivity().runOnUiThread(new Runnable() {
                                @Override
                                public void run() {
                                    sensorHubOverviewFragment.updateHumidity(humidity);
                                    if (sensorHubHumidityFragment.getView() != null)
                                        sensorHubHumidityFragment.appendDataToChart(humidity);
                                }
                            });
                        }
                    }
            );

            // Setup Optical
            mGattManager.queue(new GattCharacteristicWriteOperation(
                    mBtdevice,
                    UUID.fromString(SENSHUB_OPTIC_SERVICE_UUID),
                    UUID.fromString(SENSHUB_OPTIC_PERI_CHAR_UUID),
                    new byte[]{0x64}
            ));
            mGattManager.queue(new GattCharacteristicWriteOperation(
                    mBtdevice,
                    UUID.fromString(SENSHUB_OPTIC_SERVICE_UUID),
                    UUID.fromString(SENSHUB_OPTIC_CONF_CHAR_UUID),
                    new byte[]{0x01}
            ));
            mGattManager.queue(new GattSetNotificationOperation(
                    mBtdevice,
                    UUID.fromString(SENSHUB_OPTIC_SERVICE_UUID),
                    UUID.fromString(SENSHUB_OPTIC_DATA_CHAR_UUID),
                    UUID.fromString("00002902-0000-1000-8000-00805f9b34fb"),
                    true
            ));

            mGattManager.addCharacteristicChangeListener(
                    UUID.fromString(SENSHUB_OPTIC_DATA_CHAR_UUID),
                    new CharacteristicChangeListener() {
                        @Override
                        public void onCharacteristicChanged(String deviceAddress, BluetoothGattCharacteristic characteristic) {
                            final float optical = ByteBuffer.wrap(characteristic.getValue()).order(ByteOrder.LITTLE_ENDIAN).getFloat();
                            final float humidity = ByteBuffer.wrap(characteristic.getValue()).order(ByteOrder.LITTLE_ENDIAN).getFloat();
                            getActivity().runOnUiThread(new Runnable() {
                                @Override
                                public void run() {
                                    sensorHubOverviewFragment.updateOptical(optical);
                                    if (sensorHubOpticalFragment.getView() != null)
                                        sensorHubOpticalFragment.appendDataToChart(optical);
                                }
                            });
                        }
                    }
            );

            // Setup Pressure
            mGattManager.queue(new GattCharacteristicWriteOperation(
                    mBtdevice,
                    UUID.fromString(SENSHUB_BAROMETER_SERVICE_UUID),
                    UUID.fromString(SENSHUB_BAROMETER_PERI_CHAR_UUID),
                    new byte[]{0x64}
            ));
            mGattManager.queue(new GattCharacteristicWriteOperation(
                    mBtdevice,
                    UUID.fromString(SENSHUB_BAROMETER_SERVICE_UUID),
                    UUID.fromString(SENSHUB_BAROMETER_CONF_CHAR_UUID),
                    new byte[]{0x01}
            ));
            mGattManager.queue(new GattSetNotificationOperation(
                    mBtdevice,
                    UUID.fromString(SENSHUB_BAROMETER_SERVICE_UUID),
                    UUID.fromString(SENSHUB_BAROMETER_DATA_CHAR_UUID),
                    UUID.fromString("00002902-0000-1000-8000-00805f9b34fb"),
                    true
            ));

            mGattManager.addCharacteristicChangeListener(
                    UUID.fromString(SENSHUB_BAROMETER_DATA_CHAR_UUID),
                    new CharacteristicChangeListener() {
                        @Override
                        public void onCharacteristicChanged(String deviceAddress, BluetoothGattCharacteristic characteristic) {
                            final float pressure = ByteBuffer.wrap(characteristic.getValue()).order(ByteOrder.LITTLE_ENDIAN).getInt() / 1000.0f;
                            getActivity().runOnUiThread(new Runnable() {
                                @Override
                                public void run() {
                                    sensorHubOverviewFragment.updatePressure(pressure);
                                    if (sensorHubPressureFragment.getView() != null)
                                        sensorHubPressureFragment.appendDataToChart(pressure);
                                }
                            });
                        }
                    }
            );

            // Setup Movement
            mGattManager.queue(new GattCharacteristicWriteOperation(
                    mBtdevice,
                    UUID.fromString(SENSHUB_MOVE_SERVICE_UUID),
                    UUID.fromString(SENSHUB_MOVE_PERI_CHAR_UUID),
                    new byte[]{0x64}
            ));
            mGattManager.queue(new GattCharacteristicWriteOperation(
                    mBtdevice,
                    UUID.fromString(SENSHUB_MOVE_SERVICE_UUID),
                    UUID.fromString(SENSHUB_MOVE_CONF_CHAR_UUID),
                    new byte[]{0x01}
            ));
            mGattManager.queue(new GattSetNotificationOperation(
                    mBtdevice,
                    UUID.fromString(SENSHUB_MOVE_SERVICE_UUID),
                    UUID.fromString(SENSHUB_MOVE_DATA_CHAR_UUID),
                    UUID.fromString("00002902-0000-1000-8000-00805f9b34fb"),
                    true
            ));

            mGattManager.addCharacteristicChangeListener(
                    UUID.fromString(SENSHUB_MOVE_DATA_CHAR_UUID),
                    new CharacteristicChangeListener() {
                        @Override
                        public void onCharacteristicChanged(String deviceAddress, BluetoothGattCharacteristic characteristic) {
                            byte[] movement = characteristic.getValue();
                            byte[] accel_x_bytes = Arrays.copyOfRange(movement, 0, 2);
                            byte[] accel_y_bytes = Arrays.copyOfRange(movement, 2, 4);
                            byte[] accel_z_bytes = Arrays.copyOfRange(movement, 4, 6);

                            byte[] gyro_x_bytes = Arrays.copyOfRange(movement, 6, 8);
                            byte[] gyro_y_bytes = Arrays.copyOfRange(movement, 8, 10);
                            byte[] gyro_z_bytes = Arrays.copyOfRange(movement, 10, 12);

                            byte[] magnet_x_bytes = Arrays.copyOfRange(movement, 12, 14);
                            byte[] magnet_y_bytes = Arrays.copyOfRange(movement, 14, 16);
                            byte[] magnet_z_bytes = Arrays.copyOfRange(movement, 16, 18);

                            final short[] movement_data = new short[9];

                            movement_data[0] = ByteBuffer.wrap(gyro_x_bytes).order(ByteOrder.LITTLE_ENDIAN).getShort();
                            movement_data[1] = ByteBuffer.wrap(gyro_y_bytes).order(ByteOrder.LITTLE_ENDIAN).getShort();
                            movement_data[2] = ByteBuffer.wrap(gyro_z_bytes).order(ByteOrder.LITTLE_ENDIAN).getShort();

                            movement_data[3] = ByteBuffer.wrap(accel_x_bytes).order(ByteOrder.LITTLE_ENDIAN).getShort();
                            movement_data[4] = ByteBuffer.wrap(accel_y_bytes).order(ByteOrder.LITTLE_ENDIAN).getShort();
                            movement_data[5] = ByteBuffer.wrap(accel_z_bytes).order(ByteOrder.LITTLE_ENDIAN).getShort();

                            movement_data[6] = ByteBuffer.wrap(magnet_x_bytes).order(ByteOrder.LITTLE_ENDIAN).getShort();
                            movement_data[7] = ByteBuffer.wrap(magnet_y_bytes).order(ByteOrder.LITTLE_ENDIAN).getShort();
                            movement_data[8] = ByteBuffer.wrap(magnet_z_bytes).order(ByteOrder.LITTLE_ENDIAN).getShort();

                            getActivity().runOnUiThread(new Runnable() {
                                @Override
                                public void run() {
                                    sensorHubOverviewFragment.updateMovement(movement_data);
                                    if (sensorHubAccelerometerFragment.getView() != null)
                                        sensorHubAccelerometerFragment.appendDataToChart(movement_data);
                                    if (sensorHubGyroFragment.getView() != null)
                                        sensorHubGyroFragment.appendDataToChart(movement_data);
                                    if (sensorHubMagnetometerFragment.getView() != null)
                                        sensorHubMagnetometerFragment.appendDataToChart(movement_data);
                                }
                            });
                        }
                    }
            );
        }
    }

    @Override
    public void onResume() {
        super.onResume();
        ((AppCompatActivity)getActivity()).getSupportActionBar().setTitle("Sensors BoosterPack");
    }

    @Override
    public void onAttach(Context context) {
        super.onAttach(context);
    }

    @Override
    public void onDestroy() {
        Log.d("ProjectZeroFragment", "onDestroy()");
        super.onDestroy();
        if (mGattManager!= null) {
            mGattManager.close(mBtdevice);
            mGattManager = null;
        }
    }

    @Override
    public void onDetach() {
        super.onDetach();
    }

    /**
     * A {@link FragmentPagerAdapter} that returns a fragment corresponding to
     * one of the sections/tabs/pages.
     */
    public class SectionsPagerAdapter extends FragmentStatePagerAdapter {

        private List<Fragment> fragmentList;

        public SectionsPagerAdapter(FragmentManager fm, List<Fragment> fl) {
            super(fm);
            fragmentList = fl;
        }

        @Override
        public Fragment getItem(int position) {
            // getItem is called to instantiate the fragment for the given page.
            return fragmentList.get(position);
        }

        @Override
        public int getCount() {
            return fragmentList.size();
        }

        @Override
        public CharSequence getPageTitle(int position) {
            switch (position) {
                case 0:
                    return "Overview";
                case 1:
                    return "Thermometer";
                case 2:
                    return "Humidity";
                case 3:
                    return "Optical";
                case 4:
                    return "Pressure";
                case 5:
                    return "Accelerometer";
                case 6:
                    return "Gyro";
                case 7:
                    return "Magnetometer";
                default:
                    return "";
            }
        }
    }
}
