/* --COPYRIGHT--,BSD
 * Copyright (c) 2018, Texas Instruments Incorporated
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * *  Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * *  Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * *  Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * --/COPYRIGHT--*/

package com.example.simplelinksdkexplorer.sensorhub;

import android.bluetooth.BluetoothDevice;
import android.graphics.Color;
import android.os.Bundle;
import android.support.v4.app.Fragment;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.TextView;

import com.github.mikephil.charting.charts.LineChart;
import com.github.mikephil.charting.components.XAxis;
import com.github.mikephil.charting.data.Entry;
import com.github.mikephil.charting.data.LineData;
import com.github.mikephil.charting.data.LineDataSet;
import com.github.mikephil.charting.interfaces.datasets.ILineDataSet;
import com.example.simplelinksdkexplorer.R;
import com.example.simplelinksdkexplorer.gatt.GattManager;

import java.util.ArrayList;
import java.util.List;

public class SensorHubGyroFragment extends Fragment {

    private BluetoothDevice mBtdevice;
    private GattManager mGattManager;

    private int curIndex;
    private final int maxNumOfReadings = 20;

    private LineChart chart;
    private List<Entry> gyro_x_entries;
    private List<Entry> gyro_y_entries;
    private List<Entry> gyro_z_entries;
    private LineDataSet gyro_x_dataSet;
    private LineDataSet gyro_y_dataSet;
    private LineDataSet gyro_z_dataSet;
    private LineData lineData;

    private TextView gyro_x_textview;
    private TextView gyro_y_textview;
    private TextView gyro_z_textview;

    public SensorHubGyroFragment() {
        // Required empty public constructor
    }

    @Override
    public void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setRetainInstance(true);

        if (savedInstanceState != null) {

        } else {
            initializeData();
        }
    }

    @Override
    public View onCreateView(LayoutInflater inflater, ViewGroup container,
                             Bundle savedInstanceState) {
        // Inflate the layout for this fragment
        return inflater.inflate(R.layout.fragment_sensor_hub_gyro, container, false);
    }

    @Override
    public void onViewCreated(View view, Bundle savedInstanceState) {
        initializeChart();

        gyro_x_textview = (TextView) getActivity().findViewById(R.id.gyro_x_textview);
        gyro_y_textview = (TextView) getActivity().findViewById(R.id.gyro_y_textview);
        gyro_z_textview = (TextView) getActivity().findViewById(R.id.gyro_z_textview);
    }


    public void setBluetoothGatt(GattManager manager, BluetoothDevice btdevice) {
        mGattManager = manager;
        mBtdevice = btdevice;
    }

    private void initializeChart() {
        chart = (LineChart) getActivity().findViewById(R.id.gyro_chart);
        chart.getXAxis().setDrawGridLines(false);
        chart.getXAxis().setDrawLabels(false);
        chart.getXAxis().setPosition(XAxis.XAxisPosition.BOTH_SIDED);

        chart.getAxisLeft().setAxisMaximum(32768);
        chart.getAxisLeft().setAxisMinimum(-32768);

        chart.getAxisRight().setDrawGridLines(false);
        chart.getAxisRight().setDrawLabels(false);

        chart.getLegend().setEnabled(true);
        chart.getDescription().setText("Gyro Data from Sensors BoosterPack");
        chart.setTouchEnabled(false);
        chart.setData(lineData);
    }

    private void initializeData() {
        curIndex = 0;
        gyro_x_entries = new ArrayList<Entry>();
        gyro_y_entries = new ArrayList<Entry>();
        gyro_z_entries = new ArrayList<Entry>();
        gyro_x_dataSet = new LineDataSet(gyro_x_entries, "Gyro_X");
        gyro_y_dataSet = new LineDataSet(gyro_y_entries, "Gyro_Y");
        gyro_z_dataSet = new LineDataSet(gyro_z_entries, "Gyro_Z");
        List<ILineDataSet> dataSets = new ArrayList<ILineDataSet>();
        dataSets.add(gyro_x_dataSet);
        dataSets.add(gyro_y_dataSet);
        dataSets.add(gyro_z_dataSet);
        lineData = new LineData(dataSets);

        gyro_x_dataSet.setDrawValues(false);
        gyro_x_dataSet.setColor(Color.parseColor("Red"));
        gyro_x_dataSet.setCircleColor(Color.parseColor("Red"));
        gyro_x_dataSet.setDrawCircleHole(false);

        gyro_y_dataSet.setDrawValues(false);
        gyro_y_dataSet.setColor(Color.parseColor("Green"));
        gyro_y_dataSet.setCircleColor(Color.parseColor("Green"));
        gyro_y_dataSet.setDrawCircleHole(false);

        gyro_z_dataSet.setDrawValues(false);
        gyro_z_dataSet.setColor(Color.parseColor("Blue"));
        gyro_z_dataSet.setCircleColor(Color.parseColor("Blue"));
        gyro_z_dataSet.setDrawCircleHole(false);
    }

    public void appendDataToChart(short[] movement) {

        short gyro_x = movement[0];
        short gyro_y = movement[1];
        short gyro_z = movement[2];

        if (curIndex < maxNumOfReadings)
            curIndex++;
        else {
            for (Entry entry : gyro_x_dataSet.getValues())
                entry.setX(entry.getX()-1);
            gyro_x_dataSet.removeFirst();
            for (Entry entry : gyro_y_dataSet.getValues())
                entry.setX(entry.getX()-1);
            gyro_y_dataSet.removeFirst();
            for (Entry entry : gyro_z_dataSet.getValues())
                entry.setX(entry.getX()-1);
            gyro_z_dataSet.removeFirst();
        }
        gyro_x_dataSet.addEntry(new Entry(curIndex-1, gyro_x));
        gyro_y_dataSet.addEntry(new Entry(curIndex-1, gyro_y));
        gyro_z_dataSet.addEntry(new Entry(curIndex-1, gyro_z));

        lineData.notifyDataChanged();
        gyro_x_dataSet.setDrawValues(true);
        gyro_y_dataSet.setDrawValues(true);
        gyro_z_dataSet.setDrawValues(true);
        chart.notifyDataSetChanged();
        chart.invalidate();

        gyro_x_textview.setText(String.format("%d",gyro_x));
        gyro_y_textview.setText(String.format("%d",gyro_y));
        gyro_z_textview.setText(String.format("%d",gyro_z));
    }

}
