/* --COPYRIGHT--,BSD
 * Copyright (c) 2018, Texas Instruments Incorporated
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * *  Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * *  Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * *  Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * --/COPYRIGHT--*/

package com.example.simplelinksdkexplorer.sensorhub;

import android.bluetooth.BluetoothDevice;
import android.graphics.Color;
import android.os.Bundle;
import android.support.v4.app.Fragment;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.CompoundButton;
import android.widget.SeekBar;
import android.widget.Switch;
import android.widget.TextView;

import com.github.mikephil.charting.charts.LineChart;
import com.github.mikephil.charting.components.XAxis;
import com.github.mikephil.charting.data.Entry;
import com.github.mikephil.charting.data.LineData;
import com.github.mikephil.charting.data.LineDataSet;
import com.example.simplelinksdkexplorer.R;
import com.example.simplelinksdkexplorer.gatt.GattManager;
import com.example.simplelinksdkexplorer.gatt.GattOperationBundle;
import com.example.simplelinksdkexplorer.gatt.operations.GattCharacteristicWriteOperation;

import java.nio.ByteBuffer;
import java.nio.ByteOrder;
import java.util.ArrayList;
import java.util.List;
import java.util.UUID;

public class SensorHubHumidityFragment extends Fragment {

    private final String SENSHUB_HUMIDITY_SERVICE_UUID = "f000aa20-0451-4000-b000-000000000000";
    private final String SENSHUB_HUMIDITY_CONF_CHAR_UUID = "f000aa22-0451-4000-b000-000000000000";
    private final String SENSHUB_HUMIDITY_PERI_CHAR_UUID = "f000aa23-0451-4000-b000-000000000000";

    private BluetoothDevice mBtdevice;
    private GattManager mGattManager;

    private List<Entry> entries;
    private int curIndex;
    private final int maxNumOfReadings = 20;

    private LineChart chart;
    private LineDataSet dataSet;
    private LineData lineData;

    private float avg_humidity;
    private float max_humidity;
    private float min_humidity;

    private TextView humidity_textview;
    private TextView avg_humidity_textview;
    private TextView max_humidity_textview;
    private TextView min_humidity_textview;

    private Switch notification_switch;
    private SeekBar period_seekbar;
    private TextView period_textview;

    public SensorHubHumidityFragment() {
        // Required empty public constructor
    }

    @Override
    public void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setRetainInstance(true);

        if (savedInstanceState != null) {

        } else {
            initializeData();
        }
    }

    @Override
    public View onCreateView(LayoutInflater inflater, ViewGroup container,
                             Bundle savedInstanceState) {
        // Inflate the layout for this fragment
        return inflater.inflate(R.layout.fragment_sensor_hub_humidity, container, false);
    }

    @Override
    public void onViewCreated(View view, Bundle savedInstanceState) {
        initializeChart();

        notification_switch = (Switch)getActivity().findViewById(R.id.humidity_notification_switch);
        notification_switch.setChecked(true);
        notification_switch.setOnCheckedChangeListener(new CompoundButton.OnCheckedChangeListener() {
            @Override
            public void onCheckedChanged(CompoundButton buttonView, boolean isChecked) {
                GattOperationBundle bundle = new GattOperationBundle();
                if (isChecked) {
                    initializeData();
                    initializeChart();
                    mGattManager.queue(new GattCharacteristicWriteOperation(
                            mBtdevice, UUID.fromString(SENSHUB_HUMIDITY_SERVICE_UUID),
                            UUID.fromString(SENSHUB_HUMIDITY_CONF_CHAR_UUID),
                            new byte[]{0x01}
                    ));
                }
                else {
                    mGattManager.queue(new GattCharacteristicWriteOperation(
                            mBtdevice, UUID.fromString(SENSHUB_HUMIDITY_SERVICE_UUID),
                            UUID.fromString(SENSHUB_HUMIDITY_CONF_CHAR_UUID),
                            new byte[]{0x00}
                    ));
                }
            }
        });

        period_seekbar = (SeekBar) getActivity().findViewById(R.id.humidity_period_seekbar);
        period_textview = (TextView) getActivity().findViewById(R.id.humidity_period_textview);

        period_seekbar.setOnSeekBarChangeListener(new SeekBar.OnSeekBarChangeListener() {
            @Override
            public void onProgressChanged(SeekBar seekBar, int progress, boolean fromUser) {
                mGattManager.queue(new GattCharacteristicWriteOperation(
                        mBtdevice,
                        UUID.fromString(SENSHUB_HUMIDITY_SERVICE_UUID),
                        UUID.fromString(SENSHUB_HUMIDITY_PERI_CHAR_UUID),
                        ByteBuffer.allocate(2).order(ByteOrder.LITTLE_ENDIAN).putShort((short)((progress+1)*100)).array()
                ));
                period_textview.setText(String.format("%.1f", (progress+1)/10.0f) + "s");
            }

            @Override
            public void onStartTrackingTouch(SeekBar seekBar) {

            }

            @Override
            public void onStopTrackingTouch(SeekBar seekBar) {

            }
        });

        humidity_textview = (TextView) getActivity().findViewById(R.id.humidity_textview);
        avg_humidity_textview = (TextView) getActivity().findViewById(R.id.avg_humidity);
        max_humidity_textview = (TextView) getActivity().findViewById(R.id.max_humidity);
        min_humidity_textview = (TextView) getActivity().findViewById(R.id.min_humidity);
    }

    public void setBluetoothGatt(GattManager manager, BluetoothDevice btdevice) {
        mGattManager = manager;
        mBtdevice = btdevice;
    }

    private void initializeChart() {
        chart = (LineChart) getActivity().findViewById(R.id.humidity_chart);
        chart.getXAxis().setDrawGridLines(false);
        chart.getXAxis().setDrawLabels(false);
        chart.getXAxis().setPosition(XAxis.XAxisPosition.BOTH_SIDED);

        chart.getAxisLeft().setAxisMaximum(100);
        chart.getAxisLeft().setAxisMinimum(0);

        chart.getAxisRight().setDrawGridLines(false);
        chart.getAxisRight().setDrawLabels(false);

        chart.getLegend().setEnabled(false);
        chart.getDescription().setText("Humidity Data from Sensors BoosterPack");
        chart.setTouchEnabled(false);
        chart.setData(lineData);
    }

    private void initializeData() {
        curIndex = 0;
        entries = new ArrayList<Entry>();
        dataSet = new LineDataSet(entries, "Humidity");
        lineData = new LineData(dataSet);

        dataSet.setDrawValues(false);
        dataSet.setColor(Color.parseColor("Black"));
        dataSet.setCircleColor(Color.parseColor("Black"));
        dataSet.setDrawCircleHole(false);

        avg_humidity = 0;
        max_humidity = Float.MIN_VALUE;
        min_humidity = Float.MAX_VALUE;
    }

    public void appendDataToChart(final float humidity) {
        final String unit;
        unit = " %";

        if (curIndex < maxNumOfReadings)
            curIndex++;
        else {
            for (Entry entry : dataSet.getValues())
                entry.setX(entry.getX()-1);
            dataSet.removeFirst();
        }
        dataSet.addEntry(new Entry(curIndex-1, humidity));

        avg_humidity = 0;

        for (Entry entry : dataSet.getValues())
            avg_humidity += entry.getY();
        avg_humidity /= dataSet.getValues().size();

        if (humidity > max_humidity)
            max_humidity = humidity;
        if (humidity < min_humidity)
            min_humidity = humidity;

        humidity_textview.setText(String.format("%.1f", humidity) + unit);
        lineData.notifyDataChanged();
        dataSet.setDrawValues(true);
        chart.notifyDataSetChanged();
        chart.invalidate();

        avg_humidity_textview.setText(String.format("%.1f", avg_humidity) + unit);
        max_humidity_textview.setText(String.format("%.1f", max_humidity) + unit);
        min_humidity_textview.setText(String.format("%.1f", min_humidity) + unit);
    }

}
