/* --COPYRIGHT--,BSD
 * Copyright (c) 2018, Texas Instruments Incorporated
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * *  Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * *  Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * *  Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * --/COPYRIGHT--*/

package com.example.simplelinksdkexplorer.sensorhub;

import android.bluetooth.BluetoothDevice;
import android.graphics.Color;
import android.os.Bundle;
import android.support.v4.app.Fragment;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.TextView;

import com.github.mikephil.charting.charts.LineChart;
import com.github.mikephil.charting.components.XAxis;
import com.github.mikephil.charting.data.Entry;
import com.github.mikephil.charting.data.LineData;
import com.github.mikephil.charting.data.LineDataSet;
import com.github.mikephil.charting.interfaces.datasets.ILineDataSet;
import com.example.simplelinksdkexplorer.R;
import com.example.simplelinksdkexplorer.gatt.GattManager;

import java.util.ArrayList;
import java.util.List;

public class SensorHubMagnetometerFragment extends Fragment {

    private BluetoothDevice mBtdevice;
    private GattManager mGattManager;

    private int curIndex;
    private final int maxNumOfReadings = 20;

    private LineChart chart;
    private List<Entry> magnet_x_entries;
    private List<Entry> magnet_y_entries;
    private List<Entry> magnet_z_entries;
    private LineDataSet magnet_x_dataSet;
    private LineDataSet magnet_y_dataSet;
    private LineDataSet magnet_z_dataSet;
    private LineData lineData;

    private TextView magnet_x_textview;
    private TextView magnet_y_textview;
    private TextView magnet_z_textview;

    public SensorHubMagnetometerFragment() {
        // Required empty public constructor
    }

    @Override
    public void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setRetainInstance(true);

        if (savedInstanceState != null) {

        } else {
            initializeData();
        }
    }

    @Override
    public View onCreateView(LayoutInflater inflater, ViewGroup container,
                             Bundle savedInstanceState) {
        // Inflate the layout for this fragment
        return inflater.inflate(R.layout.fragment_sensor_hub_magnetometer, container, false);
    }

    @Override
    public void onViewCreated(View view, Bundle savedInstanceState) {
        initializeChart();

        magnet_x_textview = (TextView) getActivity().findViewById(R.id.magnet_x_textview);
        magnet_y_textview = (TextView) getActivity().findViewById(R.id.magnet_y_textview);
        magnet_z_textview = (TextView) getActivity().findViewById(R.id.magnet_z_textview);
    }


    public void setBluetoothGatt(GattManager manager, BluetoothDevice btdevice) {
        mGattManager = manager;
        mBtdevice = btdevice;
    }

    private void initializeChart() {
        chart = (LineChart) getActivity().findViewById(R.id.magnet_chart);
        chart.getXAxis().setDrawGridLines(false);
        chart.getXAxis().setDrawLabels(false);
        chart.getXAxis().setPosition(XAxis.XAxisPosition.BOTH_SIDED);

        chart.getAxisLeft().setAxisMaximum(32768);
        chart.getAxisLeft().setAxisMinimum(-32768);

        chart.getAxisRight().setDrawGridLines(false);
        chart.getAxisRight().setDrawLabels(false);

        chart.getLegend().setEnabled(true);
        chart.getDescription().setText("Magnetometer Data from Sensors BoosterPack");
        chart.setTouchEnabled(false);
        chart.setData(lineData);
    }

    private void initializeData() {
        curIndex = 0;
        magnet_x_entries = new ArrayList<Entry>();
        magnet_y_entries = new ArrayList<Entry>();
        magnet_z_entries = new ArrayList<Entry>();
        magnet_x_dataSet = new LineDataSet(magnet_x_entries, "Magnet_X");
        magnet_y_dataSet = new LineDataSet(magnet_y_entries, "Magnet_Y");
        magnet_z_dataSet = new LineDataSet(magnet_z_entries, "Magnet_Z");
        List<ILineDataSet> dataSets = new ArrayList<ILineDataSet>();
        dataSets.add(magnet_x_dataSet);
        dataSets.add(magnet_y_dataSet);
        dataSets.add(magnet_z_dataSet);
        lineData = new LineData(dataSets);

        magnet_x_dataSet.setDrawValues(false);
        magnet_x_dataSet.setColor(Color.parseColor("Red"));
        magnet_x_dataSet.setCircleColor(Color.parseColor("Red"));
        magnet_x_dataSet.setDrawCircleHole(false);

        magnet_y_dataSet.setDrawValues(false);
        magnet_y_dataSet.setColor(Color.parseColor("Green"));
        magnet_y_dataSet.setCircleColor(Color.parseColor("Green"));
        magnet_y_dataSet.setDrawCircleHole(false);

        magnet_z_dataSet.setDrawValues(false);
        magnet_z_dataSet.setColor(Color.parseColor("Blue"));
        magnet_z_dataSet.setCircleColor(Color.parseColor("Blue"));
        magnet_z_dataSet.setDrawCircleHole(false);
    }

    public void appendDataToChart(short[] movement) {

        short magnet_x = movement[6];
        short magnet_y = movement[7];
        short magnet_z = movement[8];

        if (curIndex < maxNumOfReadings)
            curIndex++;
        else {
            for (Entry entry : magnet_x_dataSet.getValues())
                entry.setX(entry.getX()-1);
            magnet_x_dataSet.removeFirst();
            for (Entry entry : magnet_y_dataSet.getValues())
                entry.setX(entry.getX()-1);
            magnet_y_dataSet.removeFirst();
            for (Entry entry : magnet_z_dataSet.getValues())
                entry.setX(entry.getX()-1);
            magnet_z_dataSet.removeFirst();
        }
        magnet_x_dataSet.addEntry(new Entry(curIndex-1, magnet_x));
        magnet_y_dataSet.addEntry(new Entry(curIndex-1, magnet_y));
        magnet_z_dataSet.addEntry(new Entry(curIndex-1, magnet_z));

        lineData.notifyDataChanged();
        magnet_x_dataSet.setDrawValues(true);
        magnet_y_dataSet.setDrawValues(true);
        magnet_z_dataSet.setDrawValues(true);
        chart.notifyDataSetChanged();
        chart.invalidate();

        magnet_x_textview.setText(String.format("%d",magnet_x));
        magnet_y_textview.setText(String.format("%d",magnet_y));
        magnet_z_textview.setText(String.format("%d",magnet_z));
    }

}
