/* --COPYRIGHT--,BSD
 * Copyright (c) 2018, Texas Instruments Incorporated
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * *  Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * *  Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * *  Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * --/COPYRIGHT--*/

package com.example.simplelinksdkexplorer.sensorhub;

import android.os.Bundle;
import android.support.v4.app.Fragment;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.TextView;

import com.example.simplelinksdkexplorer.R;

public class SensorHubOverviewFragment extends Fragment {

    private TextView ov_temperature_textview;
    private TextView ov_humidity_textview;
    private TextView ov_optical_textview;
    private TextView ov_pressure_textview;

    private TextView ov_gyro_x_textview;
    private TextView ov_gyro_y_textview;
    private TextView ov_gyro_z_textview;

    private TextView ov_accel_x_textview;
    private TextView ov_accel_y_textview;
    private TextView ov_accel_z_textview;

    private TextView ov_magnet_x_textview;
    private TextView ov_magnet_y_textview;
    private TextView ov_magnet_z_textview;

    public SensorHubOverviewFragment() {
        // Required empty public constructor
    }

    @Override
    public void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setRetainInstance(true);
    }

    @Override
    public View onCreateView(LayoutInflater inflater, ViewGroup container,
                             Bundle savedInstanceState) {
        // Inflate the layout for this fragment
        return inflater.inflate(R.layout.fragment_sensor_hub_overview, container, false);
    }

    @Override
    public void onViewCreated(View view, Bundle savedInstanceState) {
        ov_temperature_textview = (TextView) getActivity().findViewById(R.id.ov_temperature_textview);
        ov_humidity_textview = (TextView) getActivity().findViewById(R.id.ov_humidity_textview);
        ov_optical_textview = (TextView) getActivity().findViewById(R.id.ov_optical_textview);
        ov_pressure_textview = (TextView) getActivity().findViewById(R.id.ov_pressure_textview);

        ov_gyro_x_textview = (TextView) getActivity().findViewById(R.id.ov_gyro_x_textview);
        ov_gyro_y_textview = (TextView) getActivity().findViewById(R.id.ov_gyro_y_textview);
        ov_gyro_z_textview = (TextView) getActivity().findViewById(R.id.ov_gyro_z_textview);

        ov_accel_x_textview = (TextView) getActivity().findViewById(R.id.ov_accel_x_textview);
        ov_accel_y_textview = (TextView) getActivity().findViewById(R.id.ov_accel_y_textview);
        ov_accel_z_textview = (TextView) getActivity().findViewById(R.id.ov_accel_z_textview);

        ov_magnet_x_textview = (TextView) getActivity().findViewById(R.id.ov_magnet_x_textview);
        ov_magnet_y_textview = (TextView) getActivity().findViewById(R.id.ov_magnet_y_textview);
        ov_magnet_z_textview = (TextView) getActivity().findViewById(R.id.ov_magnet_z_textview);
    }

    public void updateIRTemp(final float temperature) {
        if (ov_temperature_textview != null) {
            ov_temperature_textview.setText(String.format("%.1f", temperature) + " \u2103");
        }
    }

    public void updateHumidity(final float humidity) {
        if (ov_humidity_textview != null) {
            ov_humidity_textview.setText(String.format("%.1f", humidity) + " %");
        }
    }

    public void updateOptical(final float optical) {
        if (ov_optical_textview != null) {
            ov_optical_textview.setText(String.format("%.1f", optical) + " lux");
        }
    }

    public void updatePressure(final float pressure) {
        if (ov_pressure_textview != null) {
            ov_pressure_textview.setText(String.format("%.1f", pressure) + " kPa");
        }
    }

    public void updateMovement(short[] movement) {
        if (ov_gyro_x_textview != null)
            ov_gyro_x_textview.setText("X: " + Short.toString(movement[0]));
        if (ov_gyro_y_textview != null)
            ov_gyro_y_textview.setText("Y: " + Short.toString(movement[1]));
        if (ov_gyro_z_textview != null)
            ov_gyro_z_textview.setText("Z: " + Short.toString(movement[2]));

        if (ov_accel_x_textview != null)
            ov_accel_x_textview.setText("X: " + Short.toString(movement[3]));
        if (ov_accel_y_textview != null)
            ov_accel_y_textview.setText("Y: " + Short.toString(movement[4]));
        if (ov_accel_z_textview != null)
            ov_accel_z_textview.setText("Z: " + Short.toString(movement[5]));

        if (ov_magnet_x_textview != null)
            ov_magnet_x_textview.setText("X: " + Short.toString(movement[6]));
        if (ov_magnet_y_textview != null)
            ov_magnet_y_textview.setText("Y: " + Short.toString(movement[7]));
        if (ov_magnet_z_textview != null)
            ov_magnet_z_textview.setText("Z: " + Short.toString(movement[8]));
    }

}
