/* --COPYRIGHT--,BSD
 * Copyright (c) 2018, Texas Instruments Incorporated
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * *  Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * *  Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * *  Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * --/COPYRIGHT--*/

package com.example.simplelinksdkexplorer.simpleap;

import android.bluetooth.BluetoothDevice;
import android.bluetooth.BluetoothGattCharacteristic;
import android.os.Bundle;
import android.support.v4.app.Fragment;
import android.support.v7.app.AppCompatActivity;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.Button;
import android.widget.CompoundButton;
import android.widget.EditText;
import android.widget.TextView;
import android.widget.ToggleButton;

import com.example.simplelinksdkexplorer.R;
import com.example.simplelinksdkexplorer.gatt.CharacteristicChangeListener;
import com.example.simplelinksdkexplorer.gatt.GattCharacteristicReadCallback;
import com.example.simplelinksdkexplorer.gatt.GattManager;
import com.example.simplelinksdkexplorer.gatt.operations.GattCharacteristicReadOperation;
import com.example.simplelinksdkexplorer.gatt.operations.GattCharacteristicWriteOperation;
import com.example.simplelinksdkexplorer.gatt.operations.GattSetNotificationOperation;

import java.util.UUID;

public class SimpleAPFragment extends Fragment {

    private final String SAP_SERVICE_UUID = "0000fff0-0000-1000-8000-00805f9b34fb";
    private final String SAP_LED_CHAR_UUID = "0000fff1-0000-1000-8000-00805f9b34fb";
    private final String SAP_READ_CHAR_UUID = "0000fff2-0000-1000-8000-00805f9b34fb";
    private final String SAP_WRITE_CHAR_UUID = "0000fff3-0000-1000-8000-00805f9b34fb";
    private final String SAP_NOTIFY_CHAR_UUID = "0000fff4-0000-1000-8000-00805f9b34fb";

    private static final String BT_DEVICE = "bt_device";

    private BluetoothDevice mBtdevice;
    private GattManager mGattManager;

    public SimpleAPFragment() {
        // Required empty public constructor
    }

    public static SimpleAPFragment newInstance(BluetoothDevice btdevice) {
        SimpleAPFragment fragment = new SimpleAPFragment();
        Bundle args = new Bundle();
        args.putParcelable(BT_DEVICE, btdevice);
        fragment.setArguments(args);
        return fragment;
    }

    @Override
    public void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);

        setRetainInstance(true);


        if (savedInstanceState!= null) {

        }
        else {
            if (getArguments() != null) {
                mBtdevice = getArguments().getParcelable(BT_DEVICE);
            }

            mGattManager = new GattManager(getActivity(), SAP_SERVICE_UUID);
            mGattManager.connectToDevice(mBtdevice, false);
        }
    }

    @Override
    public View onCreateView(LayoutInflater inflater, ViewGroup container,
                             Bundle savedInstanceState) {
        // Inflate the layout for this fragment
        return inflater.inflate(R.layout.fragment_simple_ap, container, false);
    }

    @Override
    public void onViewCreated(View view, Bundle savedInstanceState) {
        Button toggle_led_button = (Button) getActivity().findViewById(R.id.toggle_led_button);
        Button read_value_button = (Button) getActivity().findViewById(R.id.read_value_button);
        final TextView read_value_textview = (TextView) getActivity().findViewById(R.id.read_value_textview);
        Button write_value_button = (Button) getActivity().findViewById(R.id.write_value_button);
        final EditText write_value_edittext = (EditText) getActivity().findViewById(R.id.write_value_edittext);
        ToggleButton notification_togglebutton = (ToggleButton) getActivity().findViewById(R.id.notification_togglebutton);
        final TextView notify_value_textview = (TextView) getActivity().findViewById(R.id.notify_value_textview);

        toggle_led_button.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View view) {
                mGattManager.queue(new GattCharacteristicWriteOperation(
                        mBtdevice,
                        UUID.fromString(SAP_SERVICE_UUID),
                        UUID.fromString(SAP_LED_CHAR_UUID),
                        new byte[]{0x01}
                ));
            }
        });

        read_value_button.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View view) {
                mGattManager.queue(new GattCharacteristicReadOperation(
                        mBtdevice,
                        UUID.fromString(SAP_SERVICE_UUID),
                        UUID.fromString(SAP_READ_CHAR_UUID),
                        new GattCharacteristicReadCallback() {
                            @Override
                            public void call(byte[] characteristic) {
                                final short read_value = (short) characteristic[0];
                                getActivity().runOnUiThread(new Runnable() {
                                    @Override
                                    public void run() {
                                        read_value_textview.setText(String.format("%d", read_value));
                                    }
                                });
                            }
                        }
                ));
            }
        });

        write_value_button.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View view) {
                mGattManager.queue(new GattCharacteristicWriteOperation(
                        mBtdevice,
                        UUID.fromString(SAP_SERVICE_UUID),
                        UUID.fromString(SAP_WRITE_CHAR_UUID),
                        hexStringToByteArray(write_value_edittext.getText().toString())
                ));
            }
        });

        notification_togglebutton.setOnCheckedChangeListener(new CompoundButton.OnCheckedChangeListener() {
            @Override
            public void onCheckedChanged(CompoundButton compoundButton, boolean isChecked) {
                if (isChecked) {
                    mGattManager.queue(new GattSetNotificationOperation(
                            mBtdevice,
                            UUID.fromString(SAP_SERVICE_UUID),
                            UUID.fromString(SAP_NOTIFY_CHAR_UUID),
                            UUID.fromString("00002902-0000-1000-8000-00805f9b34fb"),
                            true
                    ));
                }
                else {
                    mGattManager.queue(new GattSetNotificationOperation(
                            mBtdevice,
                            UUID.fromString(SAP_SERVICE_UUID),
                            UUID.fromString(SAP_NOTIFY_CHAR_UUID),
                            UUID.fromString("00002902-0000-1000-8000-00805f9b34fb"),
                            false
                    ));
                }
            }
        });

        if (savedInstanceState!= null) {

        }
        else {
            mGattManager.addCharacteristicChangeListener(
                    UUID.fromString(SAP_NOTIFY_CHAR_UUID),
                    new CharacteristicChangeListener() {
                        @Override
                        public void onCharacteristicChanged(String deviceAddress, BluetoothGattCharacteristic characteristic) {
                            final short notify_value = (short) characteristic.getValue()[0];
                            getActivity().runOnUiThread(new Runnable() {
                                @Override
                                public void run() {
                                    notify_value_textview.setText(String.format("%d", notify_value));
                                }
                            });
                        }
                    }
            );
        }
    }

    @Override
    public void onResume() {
        super.onResume();
        ((AppCompatActivity) getActivity()).getSupportActionBar().setTitle("MSP432 SAP");
    }

    @Override
    public void onDestroy() {
        super.onDestroy();
        if (mGattManager!= null) {
            mGattManager.close(mBtdevice);
            mGattManager = null;
        }
    }

    public static byte[] hexStringToByteArray(String s) {
        int len = s.length();
        byte[] data = new byte[len/2];
        for (int i = 0; i < len; i += 2) {
            data[i/2] = (byte) ((Character.digit(s.charAt(i), 16) << 4)
                    + Character.digit(s.charAt(i+1), 16));
        }
        return data;
    }

}
