/* --COPYRIGHT--,BSD
 * Copyright (c) 2018, Texas Instruments Incorporated
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * *  Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * *  Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * *  Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * --/COPYRIGHT--*/

package com.example.simplelinksdkexplorer.tempsensor;

import android.bluetooth.BluetoothDevice;
import android.bluetooth.BluetoothGattCharacteristic;
import android.content.Context;
import android.graphics.Color;
import android.os.Bundle;
import android.support.v4.app.Fragment;
import android.support.v7.app.AppCompatActivity;
import android.util.Log;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.CompoundButton;
import android.widget.Switch;
import android.widget.TextView;

import com.github.mikephil.charting.charts.LineChart;
import com.github.mikephil.charting.components.XAxis;
import com.github.mikephil.charting.data.Entry;
import com.github.mikephil.charting.data.LineData;
import com.github.mikephil.charting.data.LineDataSet;
import com.example.simplelinksdkexplorer.R;
import com.example.simplelinksdkexplorer.gatt.CharacteristicChangeListener;
import com.example.simplelinksdkexplorer.gatt.GattManager;
import com.example.simplelinksdkexplorer.gatt.GattOperationBundle;
import com.example.simplelinksdkexplorer.gatt.operations.GattSetNotificationOperation;

import java.nio.ByteBuffer;
import java.nio.ByteOrder;
import java.util.ArrayList;
import java.util.List;
import java.util.UUID;

public class ThermometerFragment extends Fragment {

    private final String TEMP_SERVICE_UUID = "0000ffe0-0000-1000-8000-00805f9b34fb";
    private final String TEMP_CHARC_UUID = "0000ffe1-0000-1000-8000-00805f9b34fb";
    private final String TEMP_CHARF_UUID = "0000ffe2-0000-1000-8000-00805f9b34fb";

    private static final String BT_DEVICE = "bt_device";

    private BluetoothDevice mBtdevice;
    private GattManager mGattManager;

    private final int maxNumOfReadings = 15;
    private List<Entry> entries;
    private LineChart chart;
    private LineDataSet dataSet;
    private LineData lineData;
    private int curIndex;

    private float avg_temperature;
    private float max_temperature;
    private float min_temperature;

    public ThermometerFragment() {
        // Required empty public constructor
    }

    public static ThermometerFragment newInstance(BluetoothDevice btdevice) {
        ThermometerFragment fragment = new ThermometerFragment();
        Bundle args = new Bundle();
        args.putParcelable(BT_DEVICE, btdevice);
        fragment.setArguments(args);
        return fragment;
    }

    @Override
    public void onCreate(Bundle savedInstanceState) {
        Log.d("ThermometerFragment", "onCreate()");
        super.onCreate(savedInstanceState);

        setRetainInstance(true);


        if (savedInstanceState!= null) {

        }
        else {
            if (getArguments() != null) {
                mBtdevice = getArguments().getParcelable(BT_DEVICE);
            }

            mGattManager = new GattManager(getActivity(), TEMP_SERVICE_UUID);
            mGattManager.connectToDevice(mBtdevice, false);

            initializeData();
        }
    }

    @Override
    public View onCreateView(LayoutInflater inflater, ViewGroup container,
                             Bundle savedInstanceState) {
        Log.d("ThermometerFragment", "onCreateView()");
        // Inflate the layout for this fragment
        return inflater.inflate(R.layout.fragment_thermometer, container, false);
    }

    @Override
    public void onViewCreated(View view, Bundle savedInstanceState) {
        final Switch notification_switch = (Switch)getActivity().findViewById(R.id.notification_switch);
        final Switch unit_switch = (Switch)getActivity().findViewById(R.id.unit_switch);

        notification_switch.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View view) {
                GattOperationBundle bundle = new GattOperationBundle();
                if (notification_switch.isChecked()) {
                    initializeData();
                    initializeChart();

                    if (unit_switch.isChecked()) {
                        bundle.addOperation(new GattSetNotificationOperation(
                                mBtdevice, UUID.fromString(TEMP_SERVICE_UUID),
                                UUID.fromString(TEMP_CHARC_UUID),
                                UUID.fromString("00002902-0000-1000-8000-00805f9b34fb"),
                                false
                        ));
                        bundle.addOperation(new GattSetNotificationOperation(
                                mBtdevice, UUID.fromString(TEMP_SERVICE_UUID),
                                UUID.fromString(TEMP_CHARF_UUID),
                                UUID.fromString("00002902-0000-1000-8000-00805f9b34fb"),
                                true
                        ));
                    } else {
                        bundle.addOperation(new GattSetNotificationOperation(
                                mBtdevice, UUID.fromString(TEMP_SERVICE_UUID),
                                UUID.fromString(TEMP_CHARF_UUID),
                                UUID.fromString("00002902-0000-1000-8000-00805f9b34fb"),
                                false
                        ));
                        bundle.addOperation(new GattSetNotificationOperation(
                                mBtdevice, UUID.fromString(TEMP_SERVICE_UUID),
                                UUID.fromString(TEMP_CHARC_UUID),
                                UUID.fromString("00002902-0000-1000-8000-00805f9b34fb"),
                                true
                        ));
                    }
                }
                else {
                    bundle.addOperation(new GattSetNotificationOperation(
                            mBtdevice, UUID.fromString(TEMP_SERVICE_UUID),
                            UUID.fromString(TEMP_CHARC_UUID),
                            UUID.fromString("00002902-0000-1000-8000-00805f9b34fb"),
                            false
                    ));
                    bundle.addOperation(new GattSetNotificationOperation(
                            mBtdevice, UUID.fromString(TEMP_SERVICE_UUID),
                            UUID.fromString(TEMP_CHARF_UUID),
                            UUID.fromString("00002902-0000-1000-8000-00805f9b34fb"),
                            false
                    ));
                }
                mGattManager.queue(bundle);
            }
        });

        unit_switch.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View view) {
                GattOperationBundle bundle = new GattOperationBundle();
                if (notification_switch.isChecked()) {
                    if (unit_switch.isChecked()) {
                        bundle.addOperation(new GattSetNotificationOperation(
                                mBtdevice, UUID.fromString(TEMP_SERVICE_UUID),
                                UUID.fromString(TEMP_CHARC_UUID),
                                UUID.fromString("00002902-0000-1000-8000-00805f9b34fb"),
                                false
                        ));
                        bundle.addOperation(new GattSetNotificationOperation(
                                mBtdevice, UUID.fromString(TEMP_SERVICE_UUID),
                                UUID.fromString(TEMP_CHARF_UUID),
                                UUID.fromString("00002902-0000-1000-8000-00805f9b34fb"),
                                true
                        ));
                    } else {
                        bundle.addOperation(new GattSetNotificationOperation(
                                mBtdevice, UUID.fromString(TEMP_SERVICE_UUID),
                                UUID.fromString(TEMP_CHARF_UUID),
                                UUID.fromString("00002902-0000-1000-8000-00805f9b34fb"),
                                false
                        ));
                        bundle.addOperation(new GattSetNotificationOperation(
                                mBtdevice, UUID.fromString(TEMP_SERVICE_UUID),
                                UUID.fromString(TEMP_CHARC_UUID),
                                UUID.fromString("00002902-0000-1000-8000-00805f9b34fb"),
                                true
                        ));
                    }
                }
                mGattManager.queue(bundle);
                initializeData();
                initializeChart();
            }
        });

        if (savedInstanceState!= null) {

        }
        else {
            mGattManager.addCharacteristicChangeListener(
                    UUID.fromString(TEMP_CHARC_UUID),
                    new CharacteristicChangeListener() {
                        @Override
                        public void onCharacteristicChanged(String deviceAddress, BluetoothGattCharacteristic characteristic) {
                            float temp = ByteBuffer.wrap(characteristic.getValue()).order(ByteOrder.LITTLE_ENDIAN).getFloat();
                            appendDataToChart(temp);
                        }
                    }
            );
            mGattManager.addCharacteristicChangeListener(
                    UUID.fromString(TEMP_CHARF_UUID),
                    new CharacteristicChangeListener() {
                        @Override
                        public void onCharacteristicChanged(String deviceAddress, BluetoothGattCharacteristic characteristic) {
                            float temp = ByteBuffer.wrap(characteristic.getValue()).order(ByteOrder.LITTLE_ENDIAN).getFloat();
                            appendDataToChart(temp);
                        }
                    }
            );

            getActivity().runOnUiThread(new Runnable() {
                @Override
                public void run() {
                    ((Switch) getActivity().findViewById(R.id.notification_switch)).setChecked(true);
                    ((Switch) getActivity().findViewById(R.id.notification_switch)).callOnClick();
                }
            });
        }
    }

    @Override
    public void onAttach(Context context) {
        Log.d("ThermometerFragment", "onAttach()");
        super.onAttach(context);
    }

    @Override
    public void onDestroy() {
        Log.d("ThermometerFragment", "onDestroy()");
        super.onDestroy();
        if (mGattManager!= null) {
            mGattManager.close(mBtdevice);
            mGattManager = null;
        }
    }

    @Override
    public void onDetach() {
        Log.d("ThermometerFragment", "onDetach()");
        super.onDetach();
    }

    @Override
    public void onResume() {
        Log.d("ThermometerFragment", "onResume()");
        super.onResume();
        ((AppCompatActivity)getActivity()).getSupportActionBar().setTitle("MSP432 BLE Thermometer");

        initializeChart();
    }

    @Override
    public void onPause() {
        Log.d("ThermometerFragment", "onPause()");
        super.onPause();
    }

    private void initializeChart() {
        chart = (LineChart) getActivity().findViewById(R.id.chart);
        chart.getXAxis().setDrawGridLines(false);
        chart.getXAxis().setDrawLabels(false);
        chart.getXAxis().setPosition(XAxis.XAxisPosition.BOTH_SIDED);

        if (((Switch)getActivity().findViewById(R.id.unit_switch)).isChecked()) {
            chart.getAxisLeft().setAxisMaximum(100);
            chart.getAxisLeft().setAxisMinimum(30);
        }
        else {
            chart.getAxisLeft().setAxisMaximum(45);
            chart.getAxisLeft().setAxisMinimum(-5);
        }

        chart.getAxisRight().setDrawGridLines(false);
        chart.getAxisRight().setDrawLabels(false);

        chart.getLegend().setEnabled(false);
        chart.getDescription().setText("Temperature Data from MSP432");
        chart.setTouchEnabled(false);
        chart.setData(lineData);
    }

    private void initializeData() {
        curIndex = 0;
        entries = new ArrayList<Entry>();
        dataSet = new LineDataSet(entries, "Temperature");
        lineData = new LineData(dataSet);

        dataSet.setDrawValues(false);
        dataSet.setColor(Color.parseColor("Black"));
        dataSet.setCircleColor(Color.parseColor("Black"));
        dataSet.setDrawCircleHole(false);

        avg_temperature = 0;
        max_temperature = Float.MIN_VALUE;
        min_temperature = Float.MAX_VALUE;
    }

    private void appendDataToChart(final float temperature) {
        final String unit;
        if (((Switch)getActivity().findViewById(R.id.unit_switch)).isChecked())
            unit = " \u2109";
        else
            unit = " \u2103";

        if (curIndex < maxNumOfReadings)
            curIndex++;
        else {
            for (Entry entry : dataSet.getValues())
                entry.setX(entry.getX()-1);
            dataSet.removeFirst();
        }
        dataSet.addEntry(new Entry(curIndex-1, temperature));

        avg_temperature = 0;

        for (Entry entry : dataSet.getValues())
            avg_temperature += entry.getY();
        avg_temperature /= dataSet.getValues().size();

        if (temperature > max_temperature)
            max_temperature = temperature;
        if (temperature < min_temperature)
            min_temperature = temperature;

        getActivity().runOnUiThread(new Runnable() {
            @Override
            public void run() {
                ((TextView)getActivity().findViewById(R.id.temperature)).setText(
                        String.format("%.1f", temperature) + unit);
                lineData.notifyDataChanged();
                dataSet.setDrawValues(true);
                chart.notifyDataSetChanged();
                chart.invalidate();

                ((TextView)getActivity().findViewById(R.id.avg_temperature)).setText(
                        String.format("%.1f", avg_temperature) + unit);
                ((TextView)getActivity().findViewById(R.id.max_temperature)).setText(
                        String.format("%.1f", max_temperature) + unit);
                ((TextView)getActivity().findViewById(R.id.min_temperature)).setText(
                        String.format("%.1f", min_temperature) + unit);
            }
        });
    }

}
